<?php

namespace FiloBlu\Refilo\Remote;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterfaceFactory;
use FiloBlu\Refilo\Remote\Helper\TemplateStringFactory;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerConfigurationRepositoryInterface;
use FiloBlu\Refilo\Remote\IndexerConfiguration\IndexerResolver;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class AbstractStoreBasedIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractStoreBasedIndexer extends AbstractIndexer
{

    /**
     * @param array $ids
     * @return mixed|void
     * @throws NoSuchEntityException
     */
    public function executeAction(array $ids)
    {
        try {
            $indexTemplate = $this->getGetRemoteIndexTemplate();
        } catch (\Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
            return;
        }

        $previousStore = $this->getStoreManager()->getStore();

        foreach ($this->getStoreManager()->getStores() as $store) {
            if (!$store->getIsActive()) {
                continue;
            }

            $this->getStoreManager()->setCurrentStore($store);

            $arguments = $this->dataObjectFactory->create([
                'data' => [
                    self::ARGUMENT_COLLECTION => $indexTemplate->withValue('%s', $store->getCode())->render()
                ]
            ]);

            iterator_to_array(
                $this->entityProviderFactory->create()
                    ->forStore($store)
                    ->withIds($ids)
                    ->attachReaderHandler($this, $arguments)
            );
        }

        $this->getStoreManager()->setCurrentStore($previousStore);
    }
}
